#include "checksum.h"

#include "map.h"
#include "clientprot.h"

#include <fstream>
using namespace std;


//////// Simple Transfer checksum ////////
// Assembly rips by Coconut Emulator

BYTE simpleChecksum(void *buffer, Uint32 len)
{
	BYTE c = 0;

	for (Uint32 i = 0; i < len; ++i)
	{
		c ^= ((BYTE*)buffer)[i];
	}

	return c;
}


//////// File checksum ////////
// This section made possible by Snrrrub

Uint32 getFileChecksum(void *buffer, Uint32 *dictionary, Uint32 len)
{
	Uint32 Index = 0,
		   Key   = Index - 1;

	for (Uint32 i = 0; i < len; ++i)
	{
		Index	= dictionary[(Key & 255) ^ ((BYTE*)buffer)[i]];
		Key		= (Key >> 8) ^ Index;
	}

	return ~Key;
}

Uint32 getFileChecksum(char *fileName, Uint32 *dictionary)
{
	Uint32 chk = -1;

	ifstream file(fileName, ios::binary);
	if (file)
	{
		file.seekg (0, ios::end);
		Uint32 width = file.tellg();
		file.seekg (0, ios::beg);

		BYTE *buffer = new BYTE[width];

		file.read((char*)buffer, width);

		chk = getFileChecksum(buffer, dictionary, width);

		delete []buffer;
	}

	return chk;
}

void generate4(Uint32 *offset, Uint32 key)
{
	offset[0] = key;
	offset[1] = key ^ G4_MODIFIER;
	offset[2] = key ^= (G4_MODIFIER << 1);
	offset[3] = key ^ G4_MODIFIER;
}

void generate16(Uint32 *offset, Uint32 key)
{
	generate4(offset, key);
	generate4(offset + 4, key  ^ G16_MODIFIER);
	generate4(offset + 8, key  ^= (G16_MODIFIER << 1));
	generate4(offset + 12, key ^ G16_MODIFIER);
}

void generate64(Uint32 *offset, Uint32 key)
{
	generate16(offset, key);
	generate16(offset + 16, key ^ G64_MODIFIER);
	generate16(offset + 32, key ^= (G64_MODIFIER << 1));
	generate16(offset + 48, key ^ G64_MODIFIER);
}

void generateDictionary(Uint32 *offset, Uint32 key)
{
	generate64(offset, key);
	generate64(offset + 64, key  ^ G256_MODIFIER);
	generate64(offset + 128, key ^= (G256_MODIFIER << 1));
	generate64(offset + 192, key ^ G256_MODIFIER);
}


//////// Security checksum ////////
//  Supreme thanks go to Coconut Emulator and Snrrrub
//  for providing the assembly rips.

Uint32 generateLevelChecksum(Uint32 key, char *data)
{	// Snrrrub's interpretive version
	Uint32 numBlocks;

	Uint32 startX, endX;
	Uint32 checksum = key;
	char *mapdata = (char *)data;

	if (!data || key & 0x80000000)
		return key;

	startX = key % 0x1F;
	endX = 1024 - startX;
	mapdata += (key & 0x1F) * 1024;			// Move to the first block

	for (numBlocks = (0x41F - (key & 0x1F)) / 32; numBlocks > 0; --numBlocks, mapdata += (32 * 1024))
	{
		for (Uint32 cnt = startX; cnt < endX; cnt += 0x1F)
		{
			BYTE tile = *(mapdata + cnt);

			if (tile && (tile <= vieNormalEnd || tile == vieSafeZone))
				checksum += key ^ tile;
		}
	}

	return checksum;
}

Uint32 generateParameterChecksum(Uint32 key, Uint32 *settings)
{
	Uint32 i, c = 0;

	for (i = 0; i < 357; ++i)
	{
		c += (settings[i] ^ key);
	}

	return c;
}

Uint32 generatePartialEXE(Uint32 key, ifstream &file, Uint32 file_offset, Uint32 len)
{
	char buffer[0x0000D5A0];	// largest amount read

	file.seekg(file_offset, ios_base::beg);
	file.read(buffer, len);

	Uint32 data,
		   chk = 0,
		   offset = 0;

	while (len > 4)
	{
		data = getLong(buffer, offset);

		if ((len & 0x7FF) == 0)
			chk += (data | key);
		else
			chk ^= (data ^ key);

		len -= 3;
		offset += 3;
	}

	return chk;
}

Uint32 generateEXEChecksum(Uint32 key, char *fileName)
{
	ifstream file(fileName, ios::binary);
	if (!file) return -1;

	Uint32 chk = 0;

	chk += generatePartialEXE(key, file, 0x00006940, 0x0000D5A0);
	chk += generatePartialEXE(key, file, 0x00000400, 0x00002480);
	chk += generatePartialEXE(key, file, 0x0001CAE0, 0x00006F70);
	chk += generatePartialEXE(key, file, 0x00042E70, 0x000021E0);
	chk += generatePartialEXE(key, file, 0x000305A0, 0x0000D3C0);
	chk += generatePartialEXE(key, file, 0x0002A5B0, 0x00005FE0);
	chk += generatePartialEXE(key, file, 0x00003960, 0x00001C90);
	chk += generatePartialEXE(key, file, 0x00013EF0, 0x00008BE0);
	chk += generatePartialEXE(key, file, 0x00005750, 0x000000E0);

	return chk;
}
