/*
	MERVBot DLL-plugin API by Catid@pacbell.net
*/

#ifndef DLLCORE_H
#define DLLCORE_H

// Major version means DLL and bot are incompatible.
// Minor version means extra features are available.

/*
	5.6
		DLL2Bot EVENT_Team
		Bot2DLL	EVENT_PlayerShip + old team
		Bot2DLL	EVENT_PlayerTeam + old ship
		Bot2DLL	EVENT_PlayerSpec + old team + old ship
	5.5
		Bot2DLL	EVENT_PlayerShip + old ship
		Bot2DLL	EVENT_ArenaEnter + biller online
	5.4
		Bot2DLL	EVENT_PlayerTeam + old team
*/

#define CORE_MAJOR_VERSION 5
#define CORE_MINOR_VERSION 6

#include "player.h"
#include "commtypes.h"
#include "settings.h"

enum Bot2DLL_EventCodes
{
	EVENT_Init,
		/*	DLL has been loaded

			[0] Version hi/lo
			[1] Callback
			[2] Playerlist
			[3] Flaglist
		*/

	EVENT_Term,
		/*	DLL has been unloaded
		*/

	EVENT_Tick,
		/*	Happens every 1 second
		*/

	EVENT_ArenaEnter,
		/*	Bot has entered an arena

			[0] Name
			[1] Me Player class
			[2] Boolean: billing server online?
		*/

	EVENT_ArenaSettings,
		/*	Bot has arena settings

			[0] Settings
		*/

	EVENT_ArenaLeave,
		/*	Bot has left an arena
		*/

	EVENT_ArenaListEntry,
		/*	Entry in an incoming arena list

			[0] Name
			[1] Current?
			[2] Population
		*/

	EVENT_ArenaListEnd,
		/*	Last item of an incoming arena list

			[0] Name
			[1] Current?
			[2] Population
		*/

	EVENT_CreateTurret,
		/*	Player turreted another

			[0] Turreter
			[1] Turretee
		*/

	EVENT_DeleteTurret,
		/*	Player got off another

			[0] Turreter
			[1] Turretee
		*/

	EVENT_PlayerEntering,
		/*	Happens after the bot is in the game

			[0] Player class
		*/

	EVENT_PlayerMove,
		/*	Player changed position

			[0] Player class
		*/

	EVENT_PlayerDeath,
		/*	Player died

			[0] Player class
			[1] Killer player class
		*/

	EVENT_PlayerScore,
		/*	Player changed score

			[0] Player class
		*/

	EVENT_PlayerPrize,
		/*	Player got prized

			[0] Player class
			[1] Prize
		*/

	EVENT_PlayerShip,
		/*	Player changed ship

			[0] Player class
			[1] Old ship
			[2] Old team
		*/

	EVENT_PlayerSpec,
		/*	Player spectating

			[0] Player class
			[1] Old team
			[2] Old ship
		*/

	EVENT_PlayerTeam,
		/*	Player changed team

			[0] Player class
			[1] Old team
			[2] Old ship
		*/

	EVENT_PlayerLeaving,
		/*	Player is leaving the game.

			[0] Player class
		*/

	EVENT_Chat,
		/*	Chat message has been received.

			[0] Type
			[1] Sound
			[2] Player class/UNUSED
			[3] Message
		*/

	EVENT_LocalCommand,
		/*	Command from this arena

			[0] Player class
			[1] Command class
		*/

	EVENT_LocalHelp,
		/*	Help from this arena

			[0] Player class
			[1] Command class
		*/

	EVENT_RemoteCommand,
		/*	Command from another arena

			[0] Player name
			[1] Command class
			[2] Operator level
		*/

	EVENT_RemoteHelp,
		/*	Help from another arena

			[0] Player name
			[1] Command class
			[2] Operator level
		*/

	EVENT_FlagGrab,
		/*	Flag grabbed by a player

			[0] Player class
			[1] Flag class
		*/

	EVENT_FlagDrop,
		/*	Flags dropped by a player

			[0] Player class
		*/

	EVENT_FlagMove,
		/*	Flags moved

			[0] Flag class
		*/

	EVENT_FlagVictory,
		/*	Flag victory!

			[0] Team
			[1] Reward
		*/

	EVENT_FlagReward,
		/*	Periodic flag reward

			[0] Team
			[1] Reward
		*/

	EVENT_TimedGameOver,
		/*	Timed game over

			[0] Winner 1
			[1] Winner 2
			[2] Winner 3
			[3] Winner 4
			[4] Winner 5
		*/

	EVENT_SoccerGoal,
		/*	Soccer goal

			[0] Team
			[1] Points
		*/

	EVENT_File,
		/*	File received

			[0] Filename
		*/

	EVENT_BallMove,
		/*	Soccer ball moved

			[0] PBall struct pointer (player.h)
		*/
};

enum DLL2Bot_EventCodes
{
	EVENT_Echo,
		/*	Send a message to the terminal

			[0] message
		*/

	EVENT_Say,
		/*	Queue a chat message

			[0] type
			[1] sound
			[2] ident
			[3] message
		*/

	EVENT_Ship,
		/*	Set bot ship/spectate

			[0] ship
		*/

	EVENT_GrabFlag,
		/*	Grab a flag

			[0] flag id
		*/

	EVENT_SendPosition,
		/*	Refresh bot position on server
		*/

	EVENT_DropFlags,
		/*	Drop held flags
		*/

	EVENT_SpawnBot,
		/*	Requests that a new bot is created

			[0] name
			[1] password
			[2] staff
			[3] arena
		*/

	EVENT_ChangeArena,
		/*	Requests a change of arena

			[0] name
		*/

	EVENT_Team,
		/*	Set bot team

			[0] team
		*/
};


//////// DLL->Bot ////////

typedef void *CALL_HANDLE;
typedef void (__stdcall *CALL_COMMAND)(struct BotEvent &event);
typedef _linkedlist <Player> *CALL_PLIST;
typedef _linkedlist <Flag> *CALL_FLIST;


struct BotEvent
{
	CALL_HANDLE handle;

	int code;

	void *p[8];

	BotEvent(BotEvent &b);
	BotEvent();
};


//////// Bot->DLL ////////

BotEvent makeInit				(CALL_COMMAND c, CALL_PLIST p, CALL_FLIST f);

BotEvent makeTick				();

BotEvent makeArenaEnter			(char *name, Player *me, bool biller);
BotEvent makeArenaSettings		(arenaSettings *settings);
BotEvent makeArenaLeave			();
BotEvent makeArenaListEntry		(char *name, bool current, int population);
BotEvent makeArenaListEnd		(char *name, bool current, int population);

BotEvent makePlayerEntering		(Player *p);

BotEvent makeCreateTurret		(Player *turreter, Player *turretee);
BotEvent makeDeleteTurret		(Player *turreter, Player *turretee);

BotEvent makePlayerMove			(Player *p);
BotEvent makePlayerDeath		(Player *p, Player *k);
BotEvent makePlayerPrize		(Player *p, int prize);
BotEvent makePlayerScore		(Player *p);
BotEvent makePlayerShip			(Player *p, int oldship, int oldteam);
BotEvent makePlayerSpec			(Player *p, int oldteam, int oldship);
BotEvent makePlayerTeam			(Player *p, int oldteam, int oldship);

BotEvent makePlayerLeaving		(Player *p);

BotEvent makeFlagGrab			(Player *p, Flag *f);
BotEvent makeFlagDrop			(Player *p);
BotEvent makeFlagMove			(Flag *f);
BotEvent makeFlagVictory		(int team, int points);
BotEvent makeFlagReward			(int team, int points);

BotEvent makeTimedGameOver		(Player *p1, Player *p2, Player *p3, Player *p4, Player *p5);

BotEvent makeSoccerGoal			(int team, int points);

BotEvent makeFile				(char *name);

BotEvent makeChat				(int t, int s, Player *p, char *m);
BotEvent makeLocalHelp			(Player *p, Command *c);
BotEvent makeLocalCommand		(Player *p, Command *c);
BotEvent makeRemoteHelp			(char *p, Command *c, int o);
BotEvent makeRemoteCommand		(char *p, Command *c, int o);

BotEvent makeBallMove			(PBall *pb);

BotEvent makeTerm				();


//////// DLL->Bot ////////

BotEvent makeEcho				(char *m);
BotEvent makeSay				(int t, int s, int i, char *m);

BotEvent makeShip				(int s);
BotEvent makeTeam				(int t);
BotEvent makeGrabFlag			(int f);
BotEvent makeSendPosition		();
BotEvent makeDropFlags			();

BotEvent makeSpawnBot			(char *name, char *password, char *staff, char *arena);
BotEvent makeChangeArena		(char *name);

#endif	// DLLCORE_H
